
require 'rails_helper'
require 'vcr_config'
#require 'json'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    title: 'String',
    #siret: 'String',
    #siren: 'String',
    id_rex: 'String',
    id_cristal: 'String',
    auto_validation: 'Boolean',
    auto_request_validation: 'Boolean',
    priorities_enabled: 'Boolean',
    in_peec: 'Boolean',
    achieved_goal: 'Boolean',
    spent_on_completed_requests: 'Integer',
    spent_on_ongoing_requests: 'Integer',
    spent_on_other: 'Integer',
    annual_stock: 'Integer'
  }

  ACCEPTABLE_PARAMS = %w[
    als_id
    title
    siret
    siren
    is_root
    root_id
    auto_validation
    auto_request_validation
    priorities_enabled
    in_peec
    achieved_goal
    spent_on_completed_requests
    spent_on_ongoing_requests
    spent_on_other
    annual_stock
    total_spent
    available_points
    applicant_ids
    lessor_id
    group_id
    supervisor_group_id
    manager_group_id
  ]

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ]

  HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

  describe Api::Dmo::EntitiesController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user)}
    let!(:identity_provider) {create(:identity_provider,
      key: 'als_hermes_salarie',
      identifier_rights_url: 'https://rec.client.almerys.com/authz/realms/ALS/protocol/openid-connect/token',
      identifier_client_id: 'ALS_HERMES_BACK',
      identifier_client_secret: '77ff390e-6cca-4f09-9840-0e799a47a0ac',
      identifier_subject_issuer: 'ALS',
      account_service_url: 'https://rec.client.almerys.com/auth/realms/ALS/protocol/openid-connect/token',
      account_service_client_id: 'ALS_HERMES_SERVICE_ACCOUNT',
      account_service_client_secret: '28733e0d-c363-44a2-9f85-ea7d4784a4ad',
      create_user_url: 'https://api-rcc.be-ys.com/als-back/rcc/v1/accounts',
      add_roles_user_url: 'https://api-rcc.almerys.com/iam/api/authorization/clients/ALS_HERMES_BACK/users/{login}/roles/{role}?realm=ALS',
      create_user_headers: {
        'Content-Type': 'application/json',
        'X-GeXRT-API-Key': '169c24b0-a763-40f7-877d-7f94967ffea4',
      }
    )}
      
    describe 'GET /entities with batch of ids' do
      let!(:entity1) {create(:entity)}
      let!(:entity3) {create(:entity)}
      let!(:entity2) {create(:entity)}
      before :each do
        authenticated_header(request, responsible)
        get 'index', {ids: [entity1.id.to_s, entity2.id.to_s]}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(2)
      end
    end
      

    ACCEPTABLE_PARAMS.each do |acceptable_param|
      type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
      if type
        describe 'GET /entities with sort on field ' + acceptable_param + ' works ' do
          let!(:entity3) {create(:entity, acceptable_param => get_typed_value(type, 2))}
          let!(:entity1) {create(:entity, acceptable_param => get_typed_value(type, 0))}
          let!(:entity2) {create(:entity, acceptable_param => get_typed_value(type, 1))}
          before :each do
            authenticated_header(request, responsible)
            get 'index', {sort: {acceptable_param => 1}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            if type == 'DateTime'
              entity_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
              expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            elsif type == 'Date'
              entity_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
              expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            else
              entity_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
              expect(entity_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
            end
          end
        end
      end
    end

    describe 'GET /entities with operators' do
      let!(:entity1) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 50)}
      let!(:entity3) {create(:entity, siret: 'Super000000000', spent_on_completed_requests: 150)}
      let!(:entity2) {create(:entity, siret: 'Auper000000001', spent_on_completed_requests: 150)}
      describe 'with $lt' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {spent_on_completed_requests: {'$lt': 100}}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
      describe 'with $in' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {siret: {'$in': ['Auper000000001', 'flamand']}}
        end
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      end
      describe 'with $ne' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {spent_on_completed_requests: {'$ne': 50}}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(2)
        end
      end
      describe 'with $regex' do
        before :each do
          authenticated_header(request, responsible)
          get 'index', {siret: {'$regex': /^S/}}
        end
        it 'returns sorted results' do
          expect(response.status).to eq(200)
          expect(json['meta']['pagination']['total_objects']).to eq(2)
        end
      end
    end

    describe 'GET /entities/:id' do
      describe 'verif it work' do
        let!(:entity1) {create(:entity)}
        let!(:user_right) {create(:user_right, 
          user_id: responsible.id.to_s, 
          entity_id: entity1.id.to_s,
          type: 'entity',
          right: 'administrator',
        )}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: entity1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data']['attributes']['current_user_right']['right']).to eq('administrator')
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:entity1) {create(:entity)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: entity1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(401)
        end
      end
    end

    describe 'GET /entities/:id/user_rights' do
      let!(:entity1) {create(:entity)}
      let!(:user1) {create(:user, email: 'user1@test.com', group_ids: [entity1.manager_group_id], auth_roles:['entreprise'])}
      let!(:user2) {create(:user, email: 'user2@test.com', group_ids: [entity1.supervisor_group_id], auth_roles:['bailleur'])}
      let!(:user3) {create(:user, email: 'user3@test.com', group_ids: [entity1.group_id], auth_roles:['cre'])}
      let!(:user4) {create(:user, email: 'user4@test.com', group_ids: [entity1.group_id], auth_roles:['rdsl'])}
      
      describe 'all user rights' do
        before :each do
          authenticated_header(request, responsible)
          get 'user_rights', {id: entity1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json.length).to eq(2)
        end
      end
      describe 'for specific user' do
        before :each do
          authenticated_header(request, responsible)
          get 'user_rights', {id: entity1.id, user_id: user2.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['email']).to eq(user2.email)
          expect(json['right']).to eq('user')
        end
      end
    end

    describe 'GET /entities/:id/housing_requests_count' do
      let!(:entity) { create(:entity) }
      before :each do
        authenticated_header(request, responsible)
        get 'housing_requests_count', { id: entity.id }
      end

      it 'should return requests and delegations count' do
        expect(json['delegations']).to be_a Integer
        expect(json['in_process']).to be_a Integer
      end
    end

    context 'Superadmin searches' do
      let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'], first_name: "test_first", last_name: "test_last")}
      let!(:user1) {create(:user, auth_roles: ['demandeur'], first_name: "John", last_name: "Doe")}
      let!(:user2) {create(:user, auth_roles: ['cre'], first_name: "Jane", last_name: "Doe")}
      let!(:entity1) {create(:entity, title: "test1", siret: "12345671234567",  siren: "123456789", id_cristal: "1")}
      let!(:entity2) {create(:entity, title: "test2", siret: "22345671234567",  siren: "223456789", id_cristal: "2")}
      let!(:entity3) {create(:entity, title: "test3", siret: "32345671234567",  siren: "323456789", id_cristal: "3")}
      let!(:user_right1) {create(:user_right, user_id: user1.id, entity_id: entity1.id,  type: 'entity', right: "reader")}
      let!(:user_right2) {create(:user_right, user_id: user2.id, entity_id: entity1.id,  type: 'entity', right: "administrator")}

      describe 'GET /entities/admin_search' do
        describe "search with user that is not a Superadmin" do
          before do
            authenticated_header(request, user)
            get 'admin_search'
          end
      
          it "refuses the access" do
            body = JSON.parse(response.body)
            expect(body["code"]).to eq(401)
          end
        end
        
        describe "search with good params" do

          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              entity_params:  { siret: entity1.siret}
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(2)
            expect(datas[0]['entity_title']).to eq(entity1.title)
            expect(datas[0]['user_auth_roles']).to eq(user1.auth_roles)
          end
        end
        
        describe "search with good params + user params" do
          
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              entity_params: {title: "1"},
              user_params: {first_name: user2.first_name.downcase}
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(1)
            expect(datas[0]['entity_title']).to eq(entity1.title)
            expect(datas[0]['user_auth_roles']).to eq(user2.auth_roles)
          end
        end

        describe "search with id" do
          
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              entity_params: {id: entity1.id},
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(2)
            expect(datas[0]['entity_title']).to eq(entity1.title)
            expect(datas[0]['user_auth_roles']).to eq(user1.auth_roles)
          end
        end
      end 
  
    end 

    describe 'PUT /entities/:id/switch_manual_mode' do
      let!(:entity) {create(:entity)}
      let!(:admin) { create :user, auth_roles: ['superadministrateur']}
      before do 
        authenticated_header(request, admin)

        put 'switch_manual_mode',{
          id: entity.id 
        }
      end 

      it 'does switch manual mode' do 
        resp = JSON.parse(response.body)
        expect(resp['code']).to eq(200)
      end 
    end 
  end

  describe Api::Dmo::EntitiesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    require_relative '../../swagger_helper'
    TAGS_API_DMO_ENTITIES = 'API_DMO_ENTITIES'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as superadmin' do 

      let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'])}

      let!(:main_applicant) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "3"
        )
      }

      let!(:applicant_id) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:entity) {create(:entity, title: "test", siret: "12345671234567", id_cristal: "1")}

      let!(:housing_request) {
        create(
          :housing_request,
          main_applicant: main_applicant,
          applicant_ids: [applicant_id.id.to_s])
        
      }

      path '/api/dmo/entities/admin_search' do
        get 'get users and entities from searches params ' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'

          security [Bearer: {}]
          json_params = {}
          json_params[:entity_params] = {}
          json_params[:entity_params][:id] = {type: :string}
          json_params[:entity_params][:id_cristal] = {type: :string}
          json_params[:entity_params][:siren] = {type: :string}
          json_params[:entity_params][:siret] = {type: :string}
          json_params[:entity_params][:title] = {type: :string}
          json_params[:user_params] = {}
          json_params[:user_params][:id] = {type: :string}
          json_params[:user_params][:first_name] = {type: :string}
          json_params[:user_params][:last_name] = {type: :string}
          json_params[:user_params][:auth_roles] = {type: :string}
         
          parameter name: :params, in: :body, schema: json_params
        end
      end
    end 

    context 'as admin' do
      let!(:admin) {create(:administrator)}
      let!(:superadmin) { create(:user, auth_roles: ['superadministrateur'])}

      path '/api/dmo/entities' do
        post 'Create entity' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:applicant_ids] = {type: :array}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '201', 'entity created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
              {}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entities'
            end
          end
        end
      end

      path '/api/dmo/entities' do
        get 'Retrieves all entities' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type=''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          #end

          security [Bearer: {}]
          paginator

          response '200', 'entities found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:entities) {create_list(:entity, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {entities}
            end
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        get 'Retrieves a single entity by id' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'entity found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {entity}
            end
          end
        end
      end
      path '/api/dmo/entities/{id}/housing_requests_count' do
        get 'Get in process housing_requests and delegations count for a given entity' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'requests and delegations count' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) { create(:entity) }
            let(:id) { entity.id }

            include_context 'with integration test'
            include_context 'not containings errors'
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        put 'edit entity' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:lessor_id] = {type: :string}
          json_params[:housing_offer_ids] = {type: :array}
          json_params[:applicant_ids] = {type: :array}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '200', 'entity updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}
            let(:params) do
              jsonapi_attributes(
               {title: 'String',}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entities'
            end
          end
        end
      end

      path '/api/dmo/entities/{id}/switch_manual_mode' do
        put 'switch manual mode' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]

          parameter name: :id, in: :path, type: :string

          response '200', 'housing_objectives found' do
            let(:Authorization) {"Bearer #{token_for(superadmin)}"}
            let(:entity) {create(:entity)}
            let(:id) {entity.id}

            include_context 'not containings errors'
            include_context 'with integration test' 
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['code']).to eq(200)
            end
          end
        end
      end

      path '/api/dmo/entities/{id}' do
        delete 'Delete a single entity by id' do
          tags TAGS_API_DMO_ENTITIES
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'entity deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:entity, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end
