# frozen_string_literal: true

require 'rails_helper'

RSpec.describe Hermes::Entity, type: :model do
  it 'has a valid factory' do
    entity = FactoryBot.build(:entity)
    expect(entity).to be_valid
    expect(entity.auto_validation).to be(false)
  end

  context 'with basic entity' do
    let!(:basic_entity) { create(:entity) }

    describe 'triggered side effects are valid' do
      it 'has a group with valid name' do
        expect(basic_entity.group.name).to eq "entity_#{basic_entity.id}"
        expect(basic_entity.readers).to include "g_#{basic_entity.group.id}"
      end
    end
  end

  context 'with essential items for priority points' do
    let!(:entity1) { create(:entity, siret: '12300000000000') }

    it 'has entity1 with annual stock 8500' do
      expect(entity1.annual_stock).to be 8500
    end

    describe 'entity1 priority points attribution' do
      single_request_attributed_points = [30, 40]
      request_nb = 2

      request_nb.times do |i|
        let!(:"applicant#{i}") { create(:applicant, main_employer_siret: entity1.siret,) }
        let!(:"request#{i}") do
          create(
            :housing_request,
            is_active: true,
            main_applicant: send("applicant#{i}"),
          )
        end
      end

      it 'updates entity1 points distribution' do
        [request0, request1].each do |hr|
          hr.update(
            entity_validated: true,
            priority_points_indexes: single_request_attributed_points,
          )
        end 
        spent_points = request_nb * single_request_attributed_points.sum
        r_entity1 = entity1.reload
        expect(r_entity1.total_spent).to be(spent_points)
        expect(r_entity1.spent_on_ongoing_requests).to be(spent_points)
        expect(r_entity1.available_points).to be(8500 - spent_points)
        expect(r_entity1.spent_on_completed_requests).to be(0).or(be nil)
        expect(r_entity1.spent_on_other).to be(0).or(be nil)
      end
    end

    describe 'Geocoding' do
      let!(:geo_entity) { create(:entity, address: "vieux chemin d'arles", postal_code: '13210') }
      it 'should call geocoder if address change' do
        geo_entity.address = 'routes des palus'
        geo_entity.save!
        geo_entity.reload

        job = ActiveJob::Base.queue_adapter.enqueued_jobs.last
        expect(job[:job]).to eq(GeocodeJob)
        expect(job[:queue]).to eq('geocoder')
        expect(job[:args]).to eq(
          [
            geo_entity.class.name,
            geo_entity.id.to_s,
            true
          ]
        )
      end
    end
  end
end
